@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: MessageBatch.pm 4255 2007-12-29 21:23:21Z sysjkf $
+#   $Id: MessageBatch.pm 4442 2008-05-08 00:21:26Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use POSIX;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4255 $, 10;
+$VERSION = substr q$Revision: 4442 $, 10;
 
 my $maxcleanbytes    = 0;
 my $maxcleanmessages = 0;
@@ -71,9 +71,10 @@ my $initialised = 0;
 #       out where they will need archiving. Do the archiving once we
 #       are about to remove it from the inqueue.
 sub new {
-  my $type = shift;
-  my $lint = shift;
-  my $this = {};
+  my $type   = shift;
+  my $lint   = shift;
+  my $OnlyID = shift; # A batch only containing this message ID, if specified
+  my $this   = {};
 
   #print STDERR "In new MessageBatch\n";
 
@@ -85,7 +86,7 @@ sub new {
     bless $this, $type;
     $this->CreateEicarBatch();
   } else {
-    $global::MS->{mta}->CreateBatch($this);
+    $global::MS->{mta}->CreateBatch($this, $OnlyID);
   }
 
   $this->{starttime} = time;
@@ -249,7 +250,7 @@ sub SpamChecks {
   MailScanner::SA::CheckForCacheExpire();
 
   while(($id, $message) = each %{$this->{messages}}) {
-    next if $message->{scanmail} =~ /^[0\s]+$/;
+    next if !$message->{scanmail};
     next if $message->{deleted};
     next unless MailScanner::Config::Value('spamchecks', $message) =~ /1/;
  
@@ -290,7 +291,7 @@ sub HandleSpam {
     #print STDERR "Deleted = " . $message->{deleted} . ", isspam = " . $message->{isspam} . ", scanmail = " . $message->{scanmail} . ", spamwhitelisted = " . $message->{spamwhitelisted} . "\n";
     next if  $message->{deleted} ||
             !$message->{isspam}  ||
-             $message->{scanmail} =~ /^[0\s]+$/ ||
+            !$message->{scanmail} ||
              $message->{spamwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -312,7 +313,7 @@ sub HandleHam {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
-            $message->{scanmail} =~ /^[0\s]+$/ ||
+           !$message->{scanmail} ||
             $message->{isspam};
 
     #print STDERR "Ham checks for $id\n";
@@ -356,7 +357,7 @@ sub MCPChecks {
 
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted};
-    next if $message->{scanmail} =~ /^[0\s]+$/;
+    next if !$message->{scanmail};
     next unless MailScanner::Config::Value('mcpchecks', $message);
  
     #print STDERR "Spam checks for $id\n";
@@ -390,7 +391,7 @@ sub HandleMCP {
     # Skip deleted and non-spam messages
     next if  $message->{deleted} ||
             !$message->{ismcp}   ||
-             $message->{scanmail} =~ /^[0\s]+$/ ||
+            !$message->{scanmail} ||
              $message->{mcpwhitelisted};
 
     #print STDERR "Spam checks for $id\n";
@@ -412,7 +413,7 @@ sub HandleNonMCP {
   while(($id, $message) = each %{$this->{messages}}) {
     # Skip deleted and non-spam messages
     next if $message->{deleted} ||
-            $message->{scanmail} =~ /^[0\s]+$/ ||
+           !$message->{scanmail} ||
             $message->{ismcp};
 
     #print STDERR "Ham checks for $id\n";
@@ -435,7 +436,7 @@ sub DeliverUnscanned {
     next if $message->{deleted} || $message->{dontdeliver};
 
     # This is for mail we don't want to touch at all
-    if ($message->{scanmail} =~ /^[0\s]+$/) {
+    if (!$message->{scanmail}) {
       $OutQ = MailScanner::Config::Value('outqueuedir', $message);
       $message->DeliverUntouched($OutQ);
       $message->{deleted} = 1; # This marks it for purging from disk
@@ -476,12 +477,15 @@ sub DeliverUnscanned {
 # Parse all the messages and expand all the attachments
 sub Explode {
   my $this = shift;
+  my($Debug) = @_;
 
   my($key, $message);
 
   # jjh 2004-03-12 reap as many as we can.
   # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
-  1 until waitpid(-1, WNOHANG) == -1;
+  unless ($Debug) {
+    1 until waitpid(-1, WNOHANG) == -1;
+  }
 
   #print STDERR "About to explode messages\n";
   #print STDERR "Ignore errors about failing to find EOCD signature\n";
@@ -492,7 +496,9 @@ sub Explode {
     $message->Explode();
     # jjh 2004-03-12 reap as many as we can.
     # JKF Test 2004-11-23 1 until waitpid(-1, &POSIX::WNOHANG) == -1;
-    1 until waitpid(-1, WNOHANG) == -1;
+    unless ($Debug) {
+      1 until waitpid(-1, WNOHANG) == -1;
+    }
   }
   umask 0077;
 }
@@ -733,9 +739,11 @@ sub SignUninfected {
 
   while(($id, $message) = each %{$this->{messages}}) {
     next if $message->{deleted} || $message->{dontdeliver};
-    $message->SignUninfected()
-      if MailScanner::Config::Value('signcleanmessages', $message) &&
-         !$message->{infected};
+    if (MailScanner::Config::Value('signcleanmessages', $message) &&
+         !$message->{infected}) {
+      $message->IsAReply();
+      $message->SignUninfected();
+    }
   }
 }
 
