@@ -2,7 +2,7 @@
 #   MailScanner - SMTP E-Mail Virus Scanner
 #   Copyright (C) 2002  Julian Field
 #
-#   $Id: ZMailer.pm 4212 2007-11-20 19:59:26Z sysjkf $
+#   $Id: ZMailer.pm 4372 2008-04-02 15:45:26Z sysjkf $
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -46,7 +46,7 @@ use Encode;
 use vars qw($VERSION);
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = substr q$Revision: 4212 $, 10;
+$VERSION = substr q$Revision: 4372 $, 10;
 
 # Command-line options you need to give to sendmail to sensibly process a
 # message that is piped to it. Still need to add "-f" for specifying the
@@ -756,7 +756,7 @@ sub SendMessageEntity {
 #REVISO LEOH
 sub CreateBatch {
   my $this = shift;
-  my($batch) = @_;
+  my($batch, $onlyid) = @_;
 
   my($queuedirname, $queuedir, $MsgsInQueue, $mtime);
   my($DirtyMsgs, $DirtyBytes, $CleanMsgs, $CleanBytes);
@@ -885,6 +885,8 @@ sub CreateBatch {
       ($queuedirname, $file) = ($1,$2) if $file =~ /^(.*)\/([^\/]+)$/;
       next unless $file =~ /$mta->{HDFileRegexp}/;
       $id = $1;
+      # If they want a particular message id, ignore it if it doesn't match
+      next if $onlyid ne "" && $id ne $onlyid;
 
       #print STDERR "Adding $id to batch\n";
 
